// ACM Problem C.
// Written by Christopher Thomas.

//
// Includes
//

#include <iostream>
#include <set>
#include <sstream>

using namespace std;



//
// Prototypes
//

// Helper function for generating prime rings.
void show_rings_helper(long nmax, std::set<long> &plist,
  std::set<long> &checklist, long last, long npos, string prefix);

// Shows all prime rings for 1..nmax in lexical order.
void show_rings(long nmax, std::set<long> &plist);



//
// Functions
//

// Helper function for generating prime rings.

void show_rings_helper(long nmax, std::set<long> &plist,
  std::set<long> &checklist, long last, long nfixed, string prefix)
{
  long n;
  bool is_valid;
  long ptest;
  stringstream newprefix(stringstream::out);

  // Test all candidates.
  for (n = 1; n <= nmax; n++)
    if (0 < checklist.count(n))
    {
      // This number is available. See if it'll work.
      is_valid = true;
      ptest = n + last;
      if (1 > plist.count(ptest))
        is_valid = false;

      // Additional test if we're the last element.
      // Check against the first element (1).
      if ((nfixed + 1) >= nmax)
      {
        ptest = n + 1;
        if (1 > plist.count(ptest))
          is_valid = false;
      }

      // If we have a valid number, recurse (or emit if we're the last).
      if (is_valid)
      {
        // Emit if we're the last value needed.
        if ((nfixed + 1) >= nmax)
        {
          cout << prefix << " " << n << "\n";
        }
        else // Not the last, so recurse.
        {
          // Build the new prefix.
          newprefix.str("");
          newprefix << prefix;
          newprefix << " " << n;

          // Remove this number from the pool, recurse, then re-add.
          checklist.erase(n);
          show_rings_helper(nmax, plist, checklist,
            n, nfixed + 1, newprefix.str());
          checklist.insert(n);
        }
      }
    }
}


// Shows all prime rings for 1..nmax in lexical order.

void show_rings(long nmax, std::set<long> &plist)
{
  std::set<long> checklist;
  long n;

  // Initialize the checklist.
  checklist.clear();
  for (n = 1; n <= nmax; n++)
    checklist.insert(n);

  // We've already used 1.
  checklist.erase(1);

  // Call the helper function.
  show_rings_helper(nmax, plist, checklist, 1, 1, "1");
}



//
// Main Program
//

int main(void)
{
  long nmax;
  long cidx;
  std::set<long> plist;

  // Initialize the list of prime numbers.
  // Maximum input value is 16 + 15 = 31.
  plist.clear();
  plist.insert(2);
  plist.insert(3);
  plist.insert(5);
  plist.insert(7);
  plist.insert(11);
  plist.insert(13);
  plist.insert(17);
  plist.insert(19);
  plist.insert(23);
  plist.insert(29);
  plist.insert(31);


  // Iterate through maximum integer values.
  // FIXME - No error checking!

  cidx = 1;
  do
  {
    cin >> nmax;

    if (0 < nmax)
    {
      if (1 < cidx)
        cout << "\n";
      cout << "Case " << cidx << ":\n";

      show_rings(nmax, plist);

      cidx++;
    }
  }
  while (0 < nmax);

  // Report success.
  return 0;
}



//
// This is the end of the file.
//
